///////////////////////////////////////////////////////////////////////////////
//
//  Copyright 1998 Mitsubishi Electric Information Technology Center
//  America (MEITCA).  All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that the above copyright notice and the
//  following three paragraphs appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained from MERL - A Mitsubishi Electric Research Lab, 201
//  Broadway, Cambridge, MA 02139.
//
//  IN NO EVENT SHALL MEITCA BE LIABLE TO ANY PARTY FOR DIRECT,
//  INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
//  LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
//  DOCUMENTATION, EVEN IF MEITCA HAS BEEN ADVISED OF THE POSSIBILITY
//  OF SUCH DAMAGES.
//
//  MEITCA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//  FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON
//  AN "AS IS" BASIS, AND MEITCA HAS NO OBLIGATIONS TO PROVIDE
//  MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//  Author:
//    Brian Mirtich
//    mirtich@merl.com
//    617.621.7573
//    www.merl.com/people/mirtich
//
//  Library and API:
//    James Kuffner, Jr.
//    kuffner@stanford.edu
//    650.725.8812
//    http://robotics.stanford.edu/~kuffner
//
///////////////////////////////////////////////////////////////////////////////

#ifndef TESTAPP_H
#define TESTAPP_H

class MultiBody;
class SoTransform;
class SoNode;
class SoSensor;
class SoSeparator;


///////////////////////////////////////////////////////////////////////////////
// TestApp
///////////////////////////////////////////////////////////////////////////////
class TestApp {

public:

  // constructor / destructor

  TestApp();
  ~TestApp();

  // Add individual Link's graphics to scene graph; return pointer to
  // the SoTRansform node that controls its pose.
  SoTransform *addGraphics(SoNode *gfx, float red, float grn, float blu);

  // Multibody construction functions
  MultiBody *createPendulum();
  MultiBody *createSwimmer();
  MultiBody *createThingamajig();
  MultiBody *createChain(int numLinks);

  // Execute the main loop of the program
  void Run(char *windowTitle, int choice, int numLinks);

  // Set / get integrator timestep
  void setTimestep(Real dt) { timestep = dt; }
  const Real &getTimestep(Real dt) { return timestep; }


  // Callback routine to animate multibody
  static void move(void *data, SoSensor *);

private:

 // Data
  SoSeparator *root;               // root of scene graph
  SoTransform **sceneGraphXforms;  // xform nodes for pieces of MultiBody
  MultiBody *mb;
  Real timestep;                   // integrator step size
};

#endif  // #ifndef TESTAPP_H
